@@ -9,28 +9,35 @@ import signal
 import subprocess
 import sys
 from os import path, system, uname, remove
-from os.path import isfile
+from os.path import isfile, isdir, expanduser, basename
 from typing import Optional, List
+from urllib.parse import quote
+from click import secho
 
 from .apps import get_app_path, get_installed_apps, get_product_info, \
     get_java_path, get_path_to_latest_app, is_valid_app_path, is_toolbox_path, \
-    download_and_install
+    download_and_install, remove_app_name_files
 from .certificate_chain import get_certificate_chain
+from .defaults import save_defaults, get_path_to_defaults
+from .global_config import get_projector_server_dir, LONG_NETWORK_TIMEOUT
 from .ide_update import is_updatable_ide, get_update, update_config, check_ide_update, is_tested_ide
 from .log_utils import init_log, shutdown_log, get_path_to_log
+from .products import Product, get_all_apps
+from .projector_updates import get_latest_installer_version, is_newer_than_current, self_update
 from .secure_config import get_ca_crt_file, parse_custom_names
 
 from .utils import get_java_version, get_local_addresses
 
 from .dialogs import select_app, select_new_config_name, list_configs, \
     find_apps, edit_config, list_apps, select_installed_app, select_run_config, make_run_config, \
-    get_user_install_input, get_quick_config, select_app_path
+    get_user_install_input, get_quick_config, select_app_path, get_user_defaults
 
 from .run_config import RunConfig, get_run_configs, get_run_script_path, validate_run_config, \
     delete_config, rename_config, make_config_name, get_configs_with_app, \
-    lock_config, release_config, make_config_name_from_path
+    lock_config, release_config, make_config_name_from_path, load_config
 
 from .config_generator import save_config, check_config
+from .version import __version__
 
 
 def do_list_config(pattern: Optional[str] = None) -> None:
@@ -45,19 +52,26 @@ def do_show_config(pattern: Optional[str] = None) -> None:
     """
     run_config = select_run_config(pattern)
     print(f'Configuration: {run_config.name}')
-    print(f'IDE path: {run_config.path_to_app}')
     print(f'Projector port: {run_config.projector_port}')
+    print(f'Projector listening address: {run_config.projector_host}')
 
+    if run_config.custom_names:
+        print(f'Host name(s): {run_config.custom_names}')
+
+    print(f'IDE path: {run_config.path_to_app}')
     product_info = get_product_info(run_config.path_to_app)
-    print(f'Product info: {product_info.name}, '
+    print(f'Product name: {product_info.name}, '
           f'version={product_info.version}, '
           f'build={product_info.build_number}')
 
+    print(f'Use separate config: {"yes" if run_config.use_separate_config else "no"}')
+
     if run_config.toolbox:
         print('Toolbox config = yes')
+    else:
+        print(f'Update channel: {run_config.update_channel}')
 
-    if run_config.is_secure():
-        print('Secure config = yes')
+    print(f'Projector uses secure config (https/wss)= {"yes" if run_config.is_secure() else "no"}')
 
     if run_config.is_password_protected():
         print(f'RW Password: = {run_config.password}')
@@ -85,15 +99,13 @@ def wsl_warning() -> None:
 def get_access_urls(run_config: RunConfig) -> List[str]:
     """Returns access URLs for given config"""
 
-    schema = 'http'
-
-    if run_config.is_secure():
-        schema = 'https'
-
+    schema = 'https' if run_config.is_secure() else 'http'
     urls = []
 
     if run_config.custom_names:
         addresses = parse_custom_names(run_config.custom_names)
+    elif run_config.projector_host != RunConfig.HOST_ALL:
+        addresses = [run_config.projector_host]
     else:
         addresses = get_local_addresses()
 
@@ -104,10 +116,10 @@ def get_access_urls(run_config: RunConfig) -> List[str]:
         urls.append(f'{schema}://{address}:{run_config.projector_port}/')
 
     if run_config.password:
-        res = list(map(lambda x: x + "?token=" + run_config.password, urls))
+        res = list(map(lambda x: x + "?token=" + quote(run_config.password), urls))
 
         if run_config.password != run_config.ro_password:
-            res += list(map(lambda x: x + "?token=" + run_config.ro_password, urls))
+            res += list(map(lambda x: x + "?token=" + quote(run_config.ro_password), urls))
 
         urls = res
 
@@ -134,12 +146,30 @@ def regenerate_config_if_toolbox(run_config: RunConfig) -> RunConfig:
     return run_config
 
 
+def check_bundled_server() -> None:
+    """Checks bundled server availabiluty. If server not found fires warning and exits."""
+    if not isdir(get_projector_server_dir()):
+        message = 'No bundled server found! Exiting... \n' \
+                  'Most probably you installed or upgraded projector-installer from sources.\n' \
+                  'To resolve the issue follow this: ' \
+                  'https://github.com/JetBrains/projector-installer/blob/master/' \
+                  'README-DEV.md#Install-from-source instruction. ' \
+                  'Do not forget run `python3 setup.py bundle` BEFORE `pip install`!\n' \
+                  'Or reinstall projector-installer from pypi.'
+        secho(message, bold=True)
+
+        sys.exit(2)
+
+
 PROJECTOR_PID = None
 
 
 def do_run_config(config_name: Optional[str] = None, run_browser: bool = True) -> None:
     """Executes specified config. If given name does not specify
     config, runs interactive selection procedure."""
+
+    check_bundled_server()
+
     run_config = select_run_config(config_name)
 
     print(f'Configuration name: {run_config.name}')
@@ -160,8 +190,8 @@ def do_run_config(config_name: Optional[str] = None, run_browser: bool = True) -
         sys.exit(1)
 
     if not check_config(run_config):
-        print("WARNING: run config may not correspond to autogenerated files.\n"
-              "Try run projector config rebuild\n")
+        print('WARNING: run config may not correspond to autogenerated files.\n'
+              f'Try run projector config rebuild {run_config.name}\n')
 
     def signal_handler(*args):  # type: ignore
         # pylint: disable=unused-argument
@@ -210,8 +240,7 @@ def do_run_config(config_name: Optional[str] = None, run_browser: bool = True) -
 
 def do_add_config(hint: Optional[str], app_path: Optional[str], quick: bool) -> None:
     """
-    Adds new run config. If auto_run = True, runs it without questions.
-    Asks user otherwise.
+    Adds new run config.
     """
 
     if quick:
@@ -260,11 +289,42 @@ def do_add_config(hint: Optional[str], app_path: Optional[str], quick: bool) ->
         sys.exit(1)
 
     print(f'Adding new config with name {run_config.name}')
-
     save_config(run_config)
+    print('done.')
 
 
-def do_remove_config(config_name: Optional[str] = None) -> None:
+def do_auto_add_config(config_name: str,
+                       app_path: str,
+                       port: int,
+                       hostname: str,
+                       password: str,
+                       ro_password: str,
+                       force: bool,
+                       use_separate_config: bool) -> None:
+    """Add new Run config non-interactively"""
+
+    if config_name in get_run_configs() and not force:
+        print(f'Configuration with name {config_name} is already exist.')
+        print('Choose another name or use --force flag.')
+        sys.exit(1)
+
+    if password and not ro_password:
+        ro_password = password
+
+    run_config = RunConfig(name=config_name, path_to_app=expanduser(app_path),
+                           use_separate_config=use_separate_config,
+                           projector_port=port, projector_host=hostname,
+                           token='', password=password, ro_password=ro_password,
+                           toolbox=is_toolbox_path(app_path), custom_names=hostname)
+
+    run_config.update_channel = RunConfig.NOT_TESTED
+
+    print(f'Adding new config with name {run_config.name}')
+    save_config(run_config)
+    print('done.')
+
+
+def do_remove_config(config_name: Optional[str] = None, uninstall_ide: bool = False) -> None:
     """Selects (if necessary) and removes selected run config."""
     config_name = select_run_config(config_name).name
 
@@ -274,11 +334,17 @@ def do_remove_config(config_name: Optional[str] = None) -> None:
         print(f'Configuration {config_name} is already in use. Exiting...')
         sys.exit(1)
 
+    run_config = load_config(config_name)
+
     print(f'Removing configuration {config_name}')
-    delete_config(config_name)
     release_config(lock)
+    delete_config(config_name)
     print('done.')
 
+    if uninstall_ide and not is_toolbox_path(run_config.path_to_app):
+        app = basename(run_config.path_to_app)
+        do_uninstall_app(app)
+
 
 def do_edit_config(config_name: Optional[str] = None) -> None:
     """Selects (if necessary) and edits selected run config."""
@@ -346,7 +412,7 @@ def do_rebuild_config(config_name: Optional[str] = None) -> None:
     release_config(lock)
 
 
-def do_update_config(config_name: Optional[str] = None, allow_update: bool = False) -> None:
+def do_update_config(config_name: Optional[str] = None) -> None:
     """Updates IDE in selected config if update is available"""
     run_config = select_run_config(config_name)
 
@@ -375,7 +441,7 @@ def do_update_config(config_name: Optional[str] = None, allow_update: bool = Fal
         sys.exit(1)
 
     print(f'Updating IDE for run config {run_config.name}.')
-    update_config(run_config, product, allow_update)
+    update_config(run_config, product)
     print(f'{current} is updated to {product.name}')
 
     release_config(lock)
@@ -406,6 +472,7 @@ def do_install_cert(config_name: Optional[str], path_to_certificate: Optional[st
             print(f'Key file {path_to_key} does not exist. Exiting ...')
             sys.exit(1)
 
+        need_remove = False
         if not path_to_chain:
             path_to_chain = get_certificate_chain(path_to_certificate)
             need_remove = isfile(path_to_chain)
@@ -430,8 +497,69 @@ def do_list_app(pattern: Optional[str] = None) -> None:
     list_apps(pattern)
 
 
-def do_install_app(app_name: Optional[str], auto_run: bool = True, allow_updates: bool = False,
-                   run_browser: bool = True, quick: bool = False) -> None:
+def install_app(run_config: RunConfig, app: Product) -> RunConfig:
+    """Perform selected app install"""
+    print(f'Installing {app.name}')
+    run_config.path_to_app = download_and_install(app.url)
+
+    try:
+        validate_run_config(run_config)
+    except ValueError as exception:
+        print(f'Wrong configuration parameters: {str(exception)}, exiting ...')
+        sys.exit(1)
+
+    save_config(run_config)
+
+    return run_config
+
+
+def do_auto_install(config_name: str,
+                    app_name: str,
+                    port: Optional[int],
+                    hostname: Optional[str] = '',
+                    use_separate_config: bool = False) -> None:
+    """Performs non-interactive IDE install"""
+    configs = get_run_configs(config_name)
+
+    if config_name in configs:
+        print(f'Config with name {config_name} is already exists. Exiting...')
+        sys.exit(2)
+
+    apps = get_all_apps(pattern=app_name)
+
+    if len(apps) == 0:
+        print(f'There is no apps with name {app_name}, supported by Projector. Exiting...')
+        sys.exit(2)
+
+    if len(apps) > 1:
+        print(f'There are too many apps with name matched to {app_name}.')
+        print('Examples:')
+
+        for app in apps[:5]:
+            print(app.name)
+
+        print('Please provide more specific name. Exiting...')
+        sys.exit(2)
+
+    app = apps[0]
+    run_config = get_quick_config(config_name)
+    run_config.update_channel = RunConfig.NOT_TESTED
+    run_config.use_separate_config = use_separate_config
+
+    if port:
+        run_config.projector_port = port
+
+    if hostname:
+        run_config.custom_names = hostname
+        run_config.projector_host = hostname
+
+    install_app(run_config, app)
+
+
+def do_install_app(app_name: Optional[str],
+                   auto_run: bool = True,
+                   run_browser: bool = True,
+                   quick: bool = False) -> None:
     """Installs specified app."""
 
     if quick:
@@ -457,17 +585,8 @@ def do_install_app(app_name: Optional[str], auto_run: bool = True, allow_updates
         print('Config parameters was not specified, exiting ...')
         sys.exit(1)
 
-    print(f'Installing {app.name}')
     run_config.update_channel = channel
-    run_config.path_to_app = download_and_install(app.url, allow_updates)
-
-    try:
-        validate_run_config(run_config)
-    except ValueError as exception:
-        print(f'Wrong configuration parameters: {str(exception)}, exiting ...')
-        sys.exit(1)
-
-    save_config(run_config)
+    run_config = install_app(run_config, app)
 
     if auto_run:
         do_run_config(run_config.name, run_browser)
@@ -501,5 +620,32 @@ def do_uninstall_app(app_name: Optional[str] = None) -> None:
 
     print(f'Uninstalling {app_name}')
     app_path = get_app_path(app_name)
+    remove_app_name_files(app_name)
     shutil.rmtree(app_path)
     print('done.')
+
+
+def do_save_defaults(hostname: Optional[str]) -> None:
+    """Create and save defaults"""
+    defaults = get_user_defaults(hostname)
+    print(f'Saving defaults to {get_path_to_defaults()}')
+    save_defaults(defaults)
+    print('done.')
+
+
+def do_self_update() -> None:
+    """Do self update to latest PyPi version"""
+    pypi_version = get_latest_installer_version(LONG_NETWORK_TIMEOUT)
+
+    if pypi_version is None:
+        print('There are no available updates. Exiting...')
+        sys.exit(0)
+
+    if not is_newer_than_current(pypi_version):
+        print(f'You already running the latest projector-installer version: {__version__}. '
+              f'Exiting...')
+        sys.exit(0)
+
+    print(f'Current version is {__version__}. Upgrading projector to version {pypi_version}...')
+    self_update(pypi_version)
+    print('done.')
