@@ -10,13 +10,16 @@ from typing import Any, Optional
 import click
 
 from . import global_config
+from .apps import is_valid_app_path
 from .global_config import init_config_dir, init_cache_dir
 
 from .actions import do_install_app, do_uninstall_app, do_find_app, do_list_app, do_run_config, \
     do_list_config, do_show_config, do_add_config, do_remove_config, do_edit_config, \
-    do_rename_config, do_rebuild_config, do_install_cert, do_update_config
+    do_rename_config, do_rebuild_config, do_install_cert, do_update_config, do_auto_install, \
+    do_save_defaults, do_self_update, do_auto_add_config
 from .license import display_license
 from .projector_updates import check_for_projector_updates
+from .secure_config import is_required_ca_migration, do_ca_migration
 from .utils import expand_path
 
 
@@ -43,7 +46,9 @@ def is_cwd_exist() -> bool:
 @click.option('--cache-directory', type=click.Path(),
               default='',
               help='Path to download cache directory')
-def projector(ctx: Any, config_directory: str, cache_directory: str) -> None:
+@click.option('--accept-license', default=False, is_flag=True,
+              help='Accept GPL v2 license without prompt.')
+def projector(ctx: Any, config_directory: str, cache_directory: str, accept_license: bool) -> None:
     """
     This script helps to install, manage, and run JetBrains IDEs with Projector.
     """
@@ -53,18 +58,26 @@ def projector(ctx: Any, config_directory: str, cache_directory: str) -> None:
         print(f'Could not determine current working directory. Does {it} exist? Exiting...')
         sys.exit(1)
 
-    check_for_projector_updates()
+    if ctx.invoked_subcommand != 'self-update':
+        check_for_projector_updates()
 
     global_config.config_dir = expand_path(config_directory)
 
     if cache_directory:
         global_config.cache_dir = expand_path(cache_directory)
 
+    if is_required_ca_migration():
+        do_ca_migration()
+
     if is_first_start():
-        display_license()
+        if not accept_license:
+            display_license()
+
         init_config_dir()
-        print('Please select IDE to install:')
-        do_install_app(None, auto_run=True, allow_updates=False, run_browser=True)
+
+        if not ctx.invoked_subcommand:
+            print('Please select IDE to install:')
+            do_install_app(None, auto_run=True, run_browser=True, quick=True)
     elif not ctx.invoked_subcommand:
         click.echo(ctx.get_help())
     else:
@@ -130,24 +143,52 @@ def show(config_name: Optional[str]) -> None:
 @click.command(short_help='Add new configuration')
 @click.argument('config_name', type=click.STRING, required=False)
 @click.argument('ide_path', type=click.STRING, required=False)
+@click.option('--port', type=click.INT, required=False, help='Projector port')
+@click.option('--hostname', type=click.STRING, required=False, help='Projector hostname')
+@click.option('--password', type=click.STRING, required=False, help='Projector RW password')
+@click.option('--ro-password', type=click.STRING, required=False, help='Projector RO password')
 @click.option('--expert', default=False, is_flag=True,
               help='Expert mode - set all config parameters')
-def add(config_name: Optional[str], ide_path: Optional[str], expert: bool) -> None:
+@click.option('--force', default=False, is_flag=True,
+              help='Force add config even if config with given name is already exist')
+@click.option('--use-separate-config', default=False, is_flag=True,
+              help='Use separate configuration directory for this config. '
+                   'This option allows running multiple IDE instances.')
+def add(config_name: Optional[str],
+        ide_path: Optional[str],
+        port: Optional[int],
+        hostname: Optional[str],
+        password: Optional[str],
+        ro_password: Optional[str],
+        expert: bool,
+        force: bool,
+        use_separate_config: bool) -> None:
     """projector config add [config_name]
 
     Add a new configuration.
     """
-    do_add_config(config_name, ide_path, not expert)
+    app_path: str = ide_path if ide_path else ''
+
+    if config_name and is_valid_app_path(app_path) and port and hostname:
+        do_auto_add_config(config_name=config_name, app_path=app_path,
+                           port=port, hostname=hostname, force=force,
+                           password=password if password else '',
+                           ro_password=ro_password if ro_password else '',
+                           use_separate_config=use_separate_config)
+    else:
+        do_add_config(hint=config_name, app_path=ide_path, quick=not expert)
 
 
 @click.command(short_help='Remove configuration')
 @click.argument('config_name', type=click.STRING, required=False)
-def remove(config_name: Optional[str]) -> None:
-    """projector config remove [config_name]
+@click.option('--uninstall-ide', default=False, is_flag=True,
+              help='Also uninstall the IDE if there are no other configs for it')
+def remove(config_name: Optional[str], uninstall_ide: bool) -> None:
+    """projector config remove [config_name] [--uninstall-ide]
 
     Remove an existing configuration.
     """
-    do_remove_config(config_name)
+    do_remove_config(config_name, uninstall_ide)
 
 
 @click.command(short_help='Change existing configuration')
@@ -195,37 +236,55 @@ def run(config_name: Optional[str], run_browser: bool) -> None:
 
 @click.command(short_help='Update IDE in selected configuration')
 @click.argument('config_name', type=click.STRING, required=False)
-@click.option('--allow-updates', default=False, is_flag=True,
-              help='Allow updates of installed IDE.')
-def update(config_name: Optional[str], allow_updates: bool) -> None:
+def update(config_name: Optional[str]) -> None:
     """projector config update config_name
 
     Updates IDE in selected config if update is available
     Updates IDE in selected config if update is available
     """
-    do_update_config(config_name, allow_updates)
+    do_update_config(config_name)
 
 
-@click.command(short_help='Install and configure selected IDE')
+@click.command(short_help='Interactive install and configure selected IDE')
 @click.argument('ide_name', type=click.STRING, required=False)
 @click.option('--auto-run/--no-auto-run', default=True,
               help='Run installed IDE.')
-@click.option('--allow-updates', default=False, is_flag=True,
-              help='Allow updates of installed IDE.')
 @click.option('--run-browser/--no-browser', default=True,
               help='Auto run browser in WSL environment.')
 @click.option('--expert', default=False, is_flag=True,
               help='Expert mode - set all config parameters')
-def install_app(ide_name: Optional[str], auto_run: bool,
-                allow_updates: bool,
-                run_browser: bool, expert: bool) -> None:
+def install_app(ide_name: Optional[str],
+                auto_run: bool,
+                run_browser: bool,
+                expert: bool) -> None:
     """projector ide install [ide_name]
 
     Parameter ide_name is the name of IDE to install.
     If no IDE name is given or the pattern is ambiguous, guides the user through the
     install process.
     """
-    do_install_app(ide_name, auto_run, allow_updates, run_browser, not expert)
+    do_install_app(ide_name, auto_run, run_browser, not expert)
+
+
+@click.command(short_help='Install selected IDE')
+@click.option('--config-name', type=click.STRING, required=True, help='Name of run configuration.')
+@click.option('--ide-name', type=click.STRING, required=True, help='Name of IDE to install.')
+@click.option('--port', type=click.INT, required=False, help='Projector port')
+@click.option('--hostname', type=click.STRING, required=False, help='Projector hostname')
+@click.option('--use-separate-config', default=False, is_flag=True,
+              help='Use separate configuration directory for this installation. '
+                   'This option allows running multiple IDE instances.')
+def auto_install_app(config_name: str,
+                     ide_name: str,
+                     port: Optional[int],
+                     hostname: Optional[str],
+                     use_separate_config: bool) -> None:
+    """
+    projector ide autoinstall --config-name name --ide-name name
+    [--port listen_port] [--hostname hostname or address]
+    """
+
+    do_auto_install(config_name, ide_name, port, hostname, use_separate_config)
 
 
 @click.command(short_help='Install user certificate to given config')
@@ -243,6 +302,25 @@ def install_certificate(config_name: Optional[str], certificate: Optional[str],
     do_install_cert(config_name, certificate, key, chain)
 
 
+@click.command(short_help='Configure projector defaults')
+@click.option('--hostname', type=click.STRING, required=False)
+def defaults(hostname: Optional[str]) -> None:
+    """projector defaults [--hostname host]
+
+    Configure projector defaults
+    """
+    do_save_defaults(hostname)
+
+
+@click.command(short_help='Update projector-installer')
+def self_update() -> None:
+    """projector self-update
+
+    Update projector-installer
+    """
+    do_self_update()
+
+
 @projector.group()
 def ide() -> None:
     """
@@ -262,6 +340,7 @@ ide.add_command(find_app, name='find')
 ide.add_command(list_apps, name='list')
 ide.add_command(install_app, name='install')
 ide.add_command(uninstall, name='uninstall')
+ide.add_command(auto_install_app, name='autoinstall')
 
 # Config commands
 config.add_command(list_config, name='list')
@@ -278,4 +357,7 @@ config.add_command(update, name='update')
 projector.add_command(find_app, name='find')
 projector.add_command(run, name='run')
 projector.add_command(install_app, name='install')
+projector.add_command(auto_install_app, name='autoinstall')
 projector.add_command(install_certificate, name='install-certificate')
+projector.add_command(defaults, name='defaults')
+projector.add_command(self_update, name='self-update')
