@@ -4,19 +4,22 @@
 
 
 """User dialog related procedures."""
-import string
+import platform
 import sys
 import readline
+from getpass import getpass
 
 from os.path import expanduser
 from typing import Optional, Dict, List, Tuple, TypeVar, Callable
 
 import click
+from click import INT
 
+from .defaults import get_defaults, Defaults
 from .run_config import get_run_configs, RunConfig, get_run_config_names, get_used_projector_ports
 from .apps import get_installed_apps, get_app_path, is_toolbox_path, is_valid_app_path
 from .secure_config import generate_token
-from .utils import get_local_addresses, get_distributive_name
+from .utils import get_local_addresses
 from .products import get_compatible_apps, IDEKind, Product, get_all_apps
 
 DEF_PROJECTOR_PORT: int = 9999
@@ -59,28 +62,43 @@ def list_apps(pattern: Optional[str]) -> None:
     print_selection_list(apps)
 
 
+def get_user_input(prompt: str, default: str) -> str:
+    """Get user input string"""
+    res = input(prompt)
+
+    if not res:
+        res = default
+
+    return res
+
+
+YES_NO_STR = ['y', 'Y', 'n', 'N']
+
+
+def is_boolean_input(inp: str) -> bool:
+    """Checks if is valid boolean input"""
+    return inp in YES_NO_STR
+
+
 def ask(prompt: str, default: bool) -> bool:
     """Returns Yes-No result for given prompt.
     Contains workaround for Linux Mint issue with no-eol-on-default
     """
     question = prompt + (' [Y/n]' if default else ' [y/N]')
-    res: bool = click.prompt(question, type=bool, default=default,  # type: ignore
-                             show_default=False)
+    def_str = 'y' if default else 'n'
+    res = get_user_input(question, def_str)
 
-    if res == default and use_eol_workaround():
-        print()
+    while not is_boolean_input(res):
+        print("Invalid input, please answer y or n.")
+        res = get_user_input(question, def_str)
 
-    return res
+    return res in ('y', 'Y')
 
 
 def prompt_with_default(prompt: str, default: str) -> str:
-    """Asks for user input with default"""
-    res: str = click.prompt(prompt, default=default)
-
-    if res == default and use_eol_workaround():
-        print()
-
-    return res
+    """Get string user input with def prompt"""
+    prompt = prompt + f' [{default}]: '
+    return get_user_input(prompt, default)
 
 
 T = TypeVar('T', IDEKind, Product, str)  # pylint: disable=C0103
@@ -93,7 +111,7 @@ def select_from_list(data: List[T], name: Callable[[T], str], prompt: str) -> Op
 
     while True:
         print_selection_list(names)
-        pos: int = click.prompt(prompt, type=int)
+        pos: int = click.prompt(prompt, type=INT)
 
         if pos < 0 or pos > len(names):
             print('Invalid number.')
@@ -111,20 +129,19 @@ def select_installed_app(pattern: Optional[str] = None) -> Optional[str]:
                             'Choose an IDE number to uninstall or 0 to exit')
 
 
-def select_ide_kind() -> Optional[IDEKind]:
-    """Interactively selects desired IDE kind"""
+def select_ide_kind(pattern: Optional[str]) -> Optional[IDEKind]:
+    """Select an IDE kind that matches the given pattern or interactively select one otherwise"""
     kinds = [k for k in IDEKind if k != IDEKind.Unknown]
+    if pattern:
+        pattern = pattern.lower().split(' ')[0]
+        matched_kinds = [k for k in kinds if k.name.lower().startswith(pattern)]
+        if len(matched_kinds) == 1:
+            return matched_kinds[0]
+        if len(matched_kinds) > 1:
+            kinds = matched_kinds
     return select_from_list(kinds, lambda it: it.name, 'Choose IDE type or 0 to exit')
 
 
-NO_EOL_DISTRIBUTIVE_LIST = ['LinuxMint']
-
-
-def use_eol_workaround() -> bool:
-    """Checks if need extra eol"""
-    return get_distributive_name() in NO_EOL_DISTRIBUTIVE_LIST
-
-
 def get_app_list(kind: IDEKind, pattern: Optional[str] = None) -> Tuple[str, List[Product]]:
     """Returns compatible or full app list, depending on user choice"""
     compatible = ask('Do you want to select from Projector-tested IDE only?', default=False)
@@ -137,7 +154,7 @@ def get_app_list(kind: IDEKind, pattern: Optional[str] = None) -> Tuple[str, Lis
 
 def find_apps(pattern: Optional[str] = None) -> None:
     """Pretty-print projector-compatible applications, matched to given pattern."""
-    kind = select_ide_kind()
+    kind = select_ide_kind(pattern)
 
     if kind is None:
         print('No app kind selected, exiting...')
@@ -148,13 +165,18 @@ def find_apps(pattern: Optional[str] = None) -> None:
 
 
 def select_app(pattern: Optional[str] = None) -> Tuple[str, Optional[Product]]:
-    """Interactively selects app name from list of projector-compatible applications."""
-    kind = select_ide_kind()
+    """Selects app name from list of projector-compatible applications."""
+    kind = select_ide_kind(pattern)
 
     if kind is None:
         return RunConfig.UNKNOWN, None
 
-    channel, apps = get_app_list(kind, pattern)
+    if pattern:
+        matched_apps = get_all_apps(kind, pattern)
+        if len(matched_apps) == 1:
+            return RunConfig.NOT_TESTED, matched_apps[0]
+
+    channel, apps = get_app_list(kind)
 
     return channel, select_from_list(apps, lambda it: it.name,
                                      'Choose IDE number to install or 0 to exit')
@@ -210,7 +232,7 @@ def select_run_config(config_name: Optional[str]) -> RunConfig:
         while True:
             display_run_configs(run_configs)
             prompt = f'Choose a configuration number or 0 to exit: [0-{len(run_configs)}]'
-            config_number = click.prompt(prompt, type=int)
+            config_number = click.prompt(prompt, type=INT)
 
             if config_number < 0 or config_number > len(run_configs):
                 print('Invalid number selected.')
@@ -240,7 +262,7 @@ def select_manual_app_path(default: str = '') -> str:
     readline.parse_and_bind("tab: complete")
 
     if default:
-        prompt = f'Enter the path to IDE (default: {default}, <tab> for complete): '
+        prompt = f'Enter the path to IDE (<enter> for {default}, <tab> for complete): '
     else:
         prompt = 'Enter the path to IDE (<tab> for complete): '
 
@@ -286,9 +308,12 @@ def get_all_listening_ports() -> List[int]:
     Returns all tcp port numbers in LISTEN state (on any address).
     Reads port state from /proc/net/tcp.
     """
-    res = []
+    res: List[int] = []
 
-    with open('/proc/net/tcp', 'r') as file:
+    if platform.system() != 'Linux':
+        return res
+
+    with open('/proc/net/tcp', mode='r', encoding='utf-8') as file:
         try:
             next(file)
             for line in file:
@@ -347,26 +372,37 @@ def select_projector_port() -> int:
     return res
 
 
-ALPHABET = string.ascii_letters + string.digits
+def enter_password(prompt: str, default: str = '') -> str:
+    """Hidden password input"""
+    try:
+        password = getpass(prompt)
+    except (KeyboardInterrupt, EOFError):
+        click.echo(None)
+        sys.exit(1)
 
+    if not password and default:
+        password = default
 
-def is_valid_password(password: str) -> bool:
-    """Check password symbols for validity"""
-    return all(list(map(lambda it: ALPHABET.find(it) >= 0, password)))
+    return password
 
 
 def select_password(prompt: str, default: str = '') -> str:
-    """Prompts for password if needed """
+    """Prompts for password"""
+    matched_passwords = False
+    password: str = ''
 
-    is_valid = False
+    while not matched_passwords:
+        password = enter_password(prompt, default)
 
-    while not is_valid:
-        password: str = click.prompt(text=prompt, default=default, hide_input=True,
-                                     confirmation_prompt=True)
-        is_valid = is_valid_password(password)
+        if default and password == default:
+            return password
 
-        if not is_valid:
-            click.echo('Invalid password. Please use only alphanumeric symbols.')
+        repeat = enter_password("Repeat password:", default)
+
+        matched_passwords = password == repeat
+
+        if not matched_passwords:
+            click.echo('Password mismatch. Please try again.')
 
     return password
 
@@ -378,7 +414,6 @@ def select_password_pair(def_password: str = '', def_ro_password: str = '') -> T
     need_password = ask('Would you like to set password for connection?', default=False)
 
     if need_password:
-        click.echo('Please use only alphanumeric symbols in passwords.')
         password = select_password('Please specify RW password:', default=def_password)
 
         need_ro_password = ask('Would you like to set separate read-only password?', default=False)
@@ -391,20 +426,20 @@ def select_password_pair(def_password: str = '', def_ro_password: str = '') -> T
     return password, ro_password
 
 
-def select_custom_names(default: str = 'localhost') -> str:
+def select_host_names(default: str = '') -> str:
     """Asks user for custom domain names"""
-    use_custom_names = ask('Would you like to specify custom DNS names '
+    use_custom_names = ask('Would you like to specify hostname '
                            'for Projector access?',
                            default=False)
 
-    custom_names: str = prompt_with_default(
-        'Please specify the comma-separated list of custom names',
-        default=default) if use_custom_names else ''
+    host_names: str = prompt_with_default(
+        'Please specify the comma-separated list of host names',
+        default=get_defaults().get_host(default)) if use_custom_names else ''
 
-    return custom_names
+    return host_names
 
 
-def select_projector_host(default: str = RunConfig.HOST_ALL) -> str:
+def select_projector_listening_address(default: str = RunConfig.HOST_ALL) -> str:
     """Asks for Projector's listening address(host)"""
     use_host = ask('Would you like to specify listening address (or host) for Projector?',
                    default=False)
@@ -415,9 +450,19 @@ def select_projector_host(default: str = RunConfig.HOST_ALL) -> str:
     return res
 
 
+def select_update_channel(default: str) -> str:
+    """Select update channel"""
+    channels = [RunConfig.TESTED, RunConfig.NOT_TESTED]
+    res = select_from_list(channels, lambda it: it,
+                           f'Choose update channel or 0 to keep current({default})')
+    return default if res is None else res
+
+
 def edit_config(config: RunConfig) -> RunConfig:
     """Edits existing config."""
     config.path_to_app = select_manual_app_path(default=config.path_to_app)
+    config.use_separate_config = ask('Use separate configuration directory for this config?',
+                                     default=config.use_separate_config)
     config.toolbox = False
 
     if is_toolbox_path(config.path_to_app):
@@ -430,8 +475,8 @@ def edit_config(config: RunConfig) -> RunConfig:
         'Enter a Projector listening port (press ENTER for default)',
         default=str(config.projector_port)))
 
-    config.projector_host = select_projector_host(config.projector_host)
-    config.custom_names = select_custom_names(config.custom_names)
+    config.projector_host = select_projector_listening_address(config.projector_host)
+    config.custom_names = select_host_names(config.custom_names)
 
     keep_cert = False
 
@@ -453,6 +498,8 @@ def edit_config(config: RunConfig) -> RunConfig:
 
     config.password, config.ro_password = select_password_pair(config.password, config.ro_password)
 
+    config.update_channel = select_update_channel(config.update_channel)
+
     return config
 
 
@@ -465,6 +512,9 @@ def make_run_config(config_name: str, app_path: Optional[str] = None) -> RunConf
         print('IDE was not selected, exiting...')
         sys.exit(1)
 
+    separate_config = ask('Use separate configuration directory for this config?',
+                          default=False)
+
     is_toolbox = False
 
     if is_toolbox_path(app_path):
@@ -474,12 +524,13 @@ def make_run_config(config_name: str, app_path: Optional[str] = None) -> RunConf
             'when the app updates?', default=True)
 
     projector_port = get_def_projector_port()
-    projector_host = select_projector_host()
-    custom_names = select_custom_names()
+    projector_host = select_projector_listening_address()
+    custom_names = select_host_names()
 
     password, ro_password = select_password_pair()
 
     return RunConfig(name=config_name, path_to_app=expanduser(app_path),
+                     use_separate_config=separate_config,
                      projector_port=projector_port, projector_host=projector_host,
                      token='', password=password, ro_password=ro_password,
                      toolbox=is_toolbox, custom_names=custom_names)
@@ -488,10 +539,11 @@ def make_run_config(config_name: str, app_path: Optional[str] = None) -> RunConf
 def get_quick_config(config_name: str) -> RunConfig:
     """Generates user input in quick mode """
     return RunConfig(name=select_unused_config_name(config_name), path_to_app='',
+                     use_separate_config=False,
                      projector_port=get_def_projector_port(),
                      projector_host=RunConfig.HOST_ALL,
                      token='', password='', ro_password='',
-                     toolbox=False, custom_names='')
+                     toolbox=False, custom_names=get_defaults().host)
 
 
 def get_user_install_input(config_name_hint: str) -> Optional[RunConfig]:
@@ -501,13 +553,36 @@ def get_user_install_input(config_name_hint: str) -> Optional[RunConfig]:
     if not config_name:
         return None
 
+    separate_config = ask('Use separate configuration directory for this config?',
+                          default=False)
+
     projector_port = get_def_projector_port()
-    projector_host = select_projector_host()
-    custom_names = select_custom_names()
+    projector_host = select_projector_listening_address()
+    custom_names = select_host_names()
 
     password, ro_password = select_password_pair()
 
     return RunConfig(name=config_name, path_to_app='',
+                     use_separate_config=separate_config,
                      projector_port=projector_port, projector_host=projector_host,
                      token='', password=password, ro_password=ro_password,
                      toolbox=False, custom_names=custom_names)
+
+
+def get_user_defaults(hostname: Optional[str]) -> Defaults:
+    """Returns Defaults"""
+    defaults = get_defaults()
+
+    if hostname:
+        defaults.host = hostname
+    else:
+        use_default_host = ask('Would you like to specify default hostname '
+                               'for Projector access?',
+                               default=True)
+
+        defaults.host = prompt_with_default(
+            'Please specify default hostname for Projector '
+            '(you can specify several names, separated by comma): ',
+            default=get_defaults().get_host()) if use_default_host else ''
+
+    return defaults
