@@ -10,6 +10,7 @@ from sys import stdout
 from typing import TextIO
 
 from .global_config import get_run_configs_dir
+from .version import __version__
 
 LOG_FILE_NAME: str = 'projector.log'
 MAX_LOG_FILE_SIZE = 5 * 1024 * 1024
@@ -33,7 +34,7 @@ def restrict_log_size(config_name: str) -> None:
     size = getsize(log_name)
 
     if size > MAX_LOG_FILE_SIZE:
-        with open(log_name, 'w+') as log:
+        with open(log_name, mode='w+', encoding='utf-8') as log:
             log.seek(size - MAX_LOG_FILE_SIZE)
             content = log.read()
             log.seek(0)
@@ -45,8 +46,9 @@ def restrict_log_size(config_name: str) -> None:
 def init_log(config_name: str) -> TextIO:
     """Performs initialization of log file"""
     restrict_log_size(config_name)
-    log = open(get_path_to_log(config_name), 'a+')
-    print(f"{START_SESSION_MARK} Run config: {config_name} - {datetime.now()}", file=log)
+    log = open(get_path_to_log(config_name), mode='a+', encoding='utf-8')
+    print(f'{START_SESSION_MARK} Installer version: {__version__} Run config: {config_name} - '
+          f'{datetime.now()}', file=log)
     log.flush()
 
     return log
