@@ -5,7 +5,6 @@
 """Product class and related stuff"""
 import json
 import socket
-import sys
 from os import remove
 from os.path import join, dirname, abspath
 from tempfile import gettempdir
@@ -39,6 +38,9 @@ class IDEKind(Enum):
     PhpStorm = auto()
     WebStorm = auto()
     RubyMine = auto()
+    Rider = auto()
+    DataSpell = auto()
+    MPS = auto()
 
 
 @dataclass
@@ -87,7 +89,7 @@ def _parse_entry(entry: Any) -> Product:
 
 def load_installable_apps_from_file(file_name: str) -> List[Product]:
     """Loads installable app list from json file."""
-    with open(file_name, 'r') as file:
+    with open(file_name, mode='r', encoding='utf-8') as file:
         data = json.load(file)
 
     return [_parse_entry(entry) for entry in data]
@@ -102,7 +104,7 @@ def download_compatible_apps() -> str:
         file_name = join(gettempdir(), name)
 
         return file_name
-    except (URLError, socket.timeout):
+    except (URLError, socket.timeout, ConnectionError):
         return ''
 
 
@@ -117,20 +119,15 @@ def load_compatible_apps_from_github() -> List[Product]:
 def load_compatible_apps(file_name: str) -> List[Product]:
     """Loads from file and from github and merges results"""
     local_list = load_installable_apps_from_file(file_name)
-    github_list = load_compatible_apps_from_github()
+
+    try:
+        github_list = load_compatible_apps_from_github()
+    except (URLError, IOError):
+        github_list = []
 
     return list(set(local_list) | set(github_list))
 
 
-def init_compatible_apps() -> List[Product]:
-    """Initializes compatible apps list."""
-    try:
-        return load_compatible_apps(COMPATIBLE_IDE_FILE)
-    except IOError as error:
-        print(f'Cannot load compatible ide file: {str(error)}. Exiting...')
-        sys.exit(2)
-
-
 PRODUCTS_URL = 'https://data.services.jetbrains.com/products'
 
 KIND2CODE = {
@@ -144,24 +141,62 @@ KIND2CODE = {
     IDEKind.PhpStorm: 'PS',
     IDEKind.WebStorm: 'WS',
     IDEKind.RubyMine: 'RM',
+    IDEKind.Rider: 'RD',
+    IDEKind.DataSpell: 'PCD',
+    IDEKind.MPS: 'MPS'
 }
 
+CODE2KIND = {code: kind for kind, code in KIND2CODE.items()}
+
 # All releases before this version considered as unsupported
 EARLIEST_COMPATIBLE_VERSION = LooseVersion('2019.3')
 
 
-def get_product_releases(kind: IDEKind, timeout: float) -> List[Product]:
+def get_all_product_codes() -> str:
+    """Returns code parameter string with all known IDE codes"""
+    codes = [KIND2CODE[kind] for kind in IDEKind if kind != IDEKind.Unknown]
+    params = list(map(lambda it: f'code={it}', codes))
+
+    return '&'.join(params)
+
+
+EAP_PRODUCTS = set([IDEKind.DataSpell])
+
+
+def get_releases_url(kind: Optional[IDEKind]) -> str:
+    """Create URL for query product releases"""
+    if kind:
+        if kind in EAP_PRODUCTS:
+            return f'{PRODUCTS_URL}?code={KIND2CODE[kind]}'
+
+        return f'{PRODUCTS_URL}?code={KIND2CODE[kind]}&release.type=release'
+
+    return f'{PRODUCTS_URL}?release.type=release&{get_all_product_codes()}'
+
+
+def get_product_releases(kind: Optional[IDEKind], timeout: float) -> List[Product]:
     """Retrieves list of product releases from JB products service"""
-    url = f'{PRODUCTS_URL}?code={KIND2CODE[kind]}&release.type=release'
-    data = get_json(url, timeout=timeout)
+    url = get_releases_url(kind)
+
+    try:
+        data = get_json(url, timeout=timeout)
+    except (URLError, socket.timeout, ConnectionError):
+        data = []
+
     res = []
 
     for entry in data:
         name = entry['name']
         releases = entry['releases']
+        code = entry['code']
 
         for release in releases:
             ver = release['version']
+            release_type = release['type']
+
+            if release_type == 'eap':
+                build = release['build']
+                ver = f'{ver}.{build}'
 
             if LooseVersion(ver) < EARLIEST_COMPATIBLE_VERSION:
                 continue
@@ -172,15 +207,16 @@ def get_product_releases(kind: IDEKind, timeout: float) -> List[Product]:
                 continue
 
             link = downloads['linux']['link']
-            res.append(Product(f'{name} {ver}', link, kind, ver))
+
+            if link:
+                res.append(Product(f'{name} {ver}', link, CODE2KIND[code], ver))
 
     return res
 
 
 def get_compatible_products(kind: IDEKind) -> List[Product]:
     """Returns list of all compatible apps with given kind"""
-    apps = init_compatible_apps()
-    return [app for app in apps if app.kind == kind]
+    return [app for app in load_compatible_apps(COMPATIBLE_IDE_FILE) if app.kind == kind]
 
 
 def filter_app_by_name_pattern(data: List[Product], pattern: Optional[str] = None) -> List[Product]:
@@ -204,7 +240,7 @@ def get_compatible_apps(kind: IDEKind, pattern: Optional[str] = None) -> List[Pr
     return filter_app_by_name_pattern(apps, pattern)
 
 
-def get_all_apps(kind: IDEKind, pattern: Optional[str] = None) -> List[Product]:
+def get_all_apps(kind: Optional[IDEKind] = None, pattern: Optional[str] = None) -> List[Product]:
     """Returns list of all released apps, matched given kind and pattern."""
     apps = get_product_releases(kind, timeout=LONG_NETWORK_TIMEOUT)
     return filter_app_by_name_pattern(apps, pattern)
