@@ -6,7 +6,7 @@
 """Run configurations related functions"""
 import shutil
 from os import listdir, rename
-from os.path import join, isdir, basename
+from os.path import join, isdir, basename, isfile
 from shutil import rmtree
 from typing import Optional, Dict, List, TextIO, ClassVar
 from fcntl import lockf, LOCK_EX, LOCK_NB
@@ -15,7 +15,8 @@ from dataclasses import dataclass
 import configparser
 
 from .apps import get_app_path, get_path_to_toolbox_channel, \
-    get_app_name_from_toolbox_path, get_channel_from_toolbox_path, get_product_name
+    get_app_name_from_toolbox_path, get_channel_from_toolbox_path, \
+    get_product_name, IDEA_PROPERTIES_FILE, get_ide_properties_file, get_config_dir, get_plugins_dir
 from .global_config import get_run_configs_dir
 from .utils import create_dir_if_not_exist, generate_token
 
@@ -45,6 +46,7 @@ class RunConfig:
 
     name: str
     path_to_app: str
+    use_separate_config: bool
     projector_port: int
     token: str
     password: str
@@ -85,6 +87,32 @@ class RunConfig:
         """Returns full path to certificate chain file"""
         return join(get_path_to_certificate_dir(self.name), self.certificate_chain)
 
+    def get_path_to_idea_properties_file(self) -> str:
+        """Returns path to idea.properties"""
+
+        if self.use_separate_config:
+            return join(self.get_path(), IDEA_PROPERTIES_FILE)
+
+        return get_ide_properties_file(self.path_to_app)
+
+    def get_path_to_idea_config_dir(self) -> str:
+        """Returns path nto idea config directory"""
+        if self.use_separate_config:
+            return f'{self.get_path()}/config'
+
+        return get_config_dir(self.path_to_app)
+
+    def get_path_to_idea_plugins_dir(self) -> str:
+        """Returns path to idea plugins directory"""
+        if self.use_separate_config:
+            return f'{self.get_path()}/plugins'
+
+        return get_plugins_dir(self.path_to_app)
+
+    def get_path_to_idea_options_dir(self) -> str:
+        """Returns path to idea options directory"""
+        return join(self.get_path_to_idea_config_dir(), 'options')
+
     def _copy_cert_file(self, path_to_file: str) -> str:
         """Copy file to cert directory"""
         cert_directory = get_path_to_certificate_dir(self.name)
@@ -116,6 +144,7 @@ def load_config(config_name: str) -> RunConfig:
 
     return RunConfig(config_name,
                      config.get('IDE', 'PATH'),
+                     config.getboolean('IDE', 'USE_SEPARATE_CONFIG', fallback=False),
                      config.getint('PROJECTOR', 'PORT'),
                      config.get('SSL', 'TOKEN', fallback=''),
                      config.get('PASSWORDS', 'PASSWORD', fallback=''),
@@ -134,6 +163,12 @@ def get_run_script_path(config_name: str) -> str:
     return join(get_path_to_config(config_name), RUN_SCRIPT_NAME)
 
 
+def is_run_config_name(config_name: str) -> bool:
+    """Returns true if given config_name exist"""
+    path = get_path_to_config(config_name)
+    return isfile(join(path, CONFIG_INI_NAME))
+
+
 def get_run_configs(pattern: Optional[str] = None) -> Dict[str, RunConfig]:
     """Get run configs, matched given pattern."""
     res = {}
@@ -142,6 +177,9 @@ def get_run_configs(pattern: Optional[str] = None) -> Dict[str, RunConfig]:
         if pattern and config_name.lower().find(pattern.lower()) == -1:
             continue
 
+        if not is_run_config_name(config_name):
+            continue
+
         config = load_config(config_name)
 
         if pattern == config_name:
@@ -161,7 +199,7 @@ def get_run_config_names(pattern: Optional[str] = None) -> List[str]:
 
 def delete_config(config_name: str) -> None:
     """Removes specified config."""
-    rmtree(get_path_to_config(config_name), ignore_errors=True)
+    rmtree(get_path_to_config(config_name), ignore_errors=False)
 
 
 def rename_config(from_name: str, to_name: str) -> None:
@@ -224,7 +262,7 @@ def get_lock_file_name(config_name: str) -> str:
 
 def lock_config(config_name: str) -> Optional[TextIO]:
     """Create lock file for run config"""
-    file = open(get_lock_file_name(config_name), 'w')
+    file = open(get_lock_file_name(config_name), mode='w', encoding='utf-8')
 
     try:
         lockf(file, LOCK_EX + LOCK_NB)
