@@ -3,8 +3,11 @@
 #  in the LICENSE file.
 
 """Secure config related stuff"""
+import configparser
+import shutil
 import socket
-from os.path import join, isfile
+from os import remove
+from os.path import join, isfile, isdir
 from typing import List, Tuple, Optional, TextIO
 
 import re
@@ -13,13 +16,130 @@ import subprocess
 from .global_config import get_ssl_dir, get_ssl_properties_file
 from .log_utils import init_log, shutdown_log
 from .utils import create_dir_if_not_exist, remove_file_if_exist, \
-    get_local_addresses, generate_token
+    get_local_addresses, generate_token, is_linux_x86_64
 from .apps import get_jre_dir
-from .run_config import RunConfig, get_path_to_config
+from .run_config import RunConfig, get_path_to_config, get_run_configs
 
 PROJECTOR_JKS_NAME = 'projector'
 CA_NAME = 'ca'
-CA_PASSWORD = '85TibAyPS3NZX3'
+DEF_CA_SEZAM_LEGACY = '85TibAyPS3NZX3'
+
+
+def is_required_ca_migration() -> bool:
+    """Returns True is it's necessary to migrate ca to new format"""
+    return isdir(get_ssl_dir()) and not isfile(get_ca_ini_file()) and len(get_run_configs()) > 0
+
+
+def do_ca_migration() -> bool:
+    """Migrate ca to new format"""
+    ret = make_ca_backup()
+    ret = ret and do_migration()
+
+    if ret:
+        remove_ca_backup()
+    else:
+        restore_ca_backup()
+
+    return ret
+
+
+def do_migration() -> bool:
+    """Do actual migration"""
+    token = generate_token()
+    ret = change_ca_passwords(token)
+
+    if ret:
+        create_ca_ini(token)
+
+    return ret
+
+
+def change_ca_passwords(token: str) -> bool:
+    """Change ca storage passwords"""
+    run_configs = get_run_configs()
+
+    if len(run_configs) == 0:
+        return False
+
+    config: RunConfig = next(iter(run_configs.items()))[1]
+    keytool_path = get_keytool(config.path_to_app)
+
+    cmd = [keytool_path, '-storepasswd', '-new', token, '-keystore', get_ca_jks_file(),
+           '-storepass', DEF_CA_SEZAM_LEGACY]
+
+    try:
+        subprocess.check_call(cmd, stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
+    except subprocess.CalledProcessError:
+        return False
+
+    cmd = [keytool_path, '-keypasswd', '-alias', CA_NAME, '-keypass', DEF_CA_SEZAM_LEGACY,
+           '-new', token, '-keystore', get_ca_jks_file(), '-storepass', token]
+
+    try:
+        subprocess.check_call(cmd, stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
+    except subprocess.CalledProcessError:
+        return False
+
+    return True
+
+
+def get_ca_ini_file() -> str:
+    """Return path to ca.ini"""
+    return join(get_ssl_dir(), f'{CA_NAME}.ini')
+
+
+def create_ca_ini(token: str) -> None:
+    """Create CA ini file"""
+    config = configparser.ConfigParser(strict=False, interpolation=None)
+    config['CA'] = {}
+    config['CA']['SAVED'] = token
+    with open(get_ca_ini_file(), mode='w', encoding='utf-8') as configfile:
+        config.write(configfile)
+
+
+def get_ca_jks_backup_file() -> str:
+    """Returns name of ca backup file"""
+    return join(get_ssl_dir(), f'{CA_NAME}.jks.backup')
+
+
+def make_ca_backup() -> bool:
+    """Make backup copy of ca file"""
+    try:
+        shutil.copy(get_ca_jks_file(), get_ca_jks_backup_file())
+    except OSError:
+        return False
+
+    return True
+
+
+def restore_ca_backup() -> bool:
+    """Restore ca from backup copy"""
+    try:
+        shutil.copy(get_ca_jks_backup_file(), get_ca_jks_file())
+    except OSError:
+        return False
+
+    return True
+
+
+def remove_ca_backup() -> bool:
+    """Remove ca backup file"""
+    try:
+        remove(get_ca_jks_backup_file())
+    except OSError:
+        return False
+
+    return True
+
+
+def get_ca_password() -> str:
+    """Return CA password"""
+    if isfile(get_ca_ini_file()):
+        config = configparser.ConfigParser(strict=False, interpolation=None)
+        config.read(get_ca_ini_file())
+        return config.get('CA', 'SAVED')
+
+    return DEF_CA_SEZAM_LEGACY
 
 
 def get_projector_jks_file(config_name: str) -> str:
@@ -64,7 +184,7 @@ def get_generate_ca_command() -> List[str]:
     """Returns list of args for generate ca"""
     return ['-genkeypair', '-alias', CA_NAME,
             '-dname', get_ca_dist_name(), '-keystore', get_ca_jks_file(),
-            '-keypass', CA_PASSWORD, '-storepass', CA_PASSWORD,
+            '-keypass', get_ca_password(), '-storepass', get_ca_password(),
             '-keyalg', 'RSA', '-keysize', '4096',
             '-ext', 'KeyUsage:critical=keyCertSign',
             '-ext', 'BasicConstraints:critical=ca:true',
@@ -74,8 +194,8 @@ def get_generate_ca_command() -> List[str]:
 
 def get_export_ca_command() -> List[str]:
     """Returns list of args for export ca.crt"""
-    return ['-export', '-alias', CA_NAME, '-file', get_ca_crt_file(), '-keypass', CA_PASSWORD,
-            '-storepass', CA_PASSWORD, '-keystore', get_ca_jks_file(), '-rfc']
+    return ['-export', '-alias', CA_NAME, '-file', get_ca_crt_file(), '-keypass', get_ca_password(),
+            '-storepass', get_ca_password(), '-keystore', get_ca_jks_file(), '-rfc']
 
 
 DIST_PROJECTOR_NAME = 'CN=Idea, OU=Development, O=Idea, L=SPB, S=SPB, C=RU'
@@ -87,7 +207,7 @@ def get_projector_gen_jks_args(run_config: RunConfig) -> List[str]:
         '-genkeypair', '-alias', PROJECTOR_JKS_NAME, '-dname', DIST_PROJECTOR_NAME,
         '-keystore', get_projector_jks_file(run_config.name),
         '-keypass', run_config.token, '-storepass', run_config.token,
-        '-keyalg', 'RSA', '-keysize', '4096', '-validity', '4500'
+        '-keyalg', 'RSA', '-keysize', '4096', '-validity', '9999'
     ]
 
 
@@ -97,7 +217,7 @@ def get_projector_cert_sign_request_args(run_config: RunConfig) -> List[str]:
         '-certreq', '-alias', PROJECTOR_JKS_NAME, '-keypass', run_config.token,
         '-storepass', run_config.token,
         '-keystore', get_projector_jks_file(run_config.name),
-        '-file', get_projector_csr_file(run_config.name)
+        '-file', get_projector_csr_file(run_config.name), '-validity', '9999'
     ]
 
 
@@ -155,14 +275,14 @@ def get_projector_cert_sign_args(run_config: RunConfig) -> List[str]:
     return [
         '-gencert',
         '-alias', CA_NAME,
-        '-storepass', CA_PASSWORD,
+        '-storepass', get_ca_password(),
         '-keystore', get_ca_jks_file(),
         '-infile', get_projector_csr_file(run_config.name),
         '-outfile', get_projector_crt_file(run_config.name),
         '-ext', 'KeyUsage:critical=digitalSignature,keyEncipherment',
         '-ext', 'EKU=serverAuth',
         '-ext', f'SAN={get_projector_san("0.0.0.0", run_config.custom_names)}',
-        '-rfc'
+        '-rfc', '-validity', '9999'
     ]
 
 
@@ -191,7 +311,7 @@ def get_projector_import_cert_args(run_config: RunConfig) -> List[str]:
 
 def generate_ssl_properties_file(config_name: str, token: str) -> None:
     """Generates ssl.properties file for given config"""
-    with open(get_ssl_properties_file(config_name), "w") as file:
+    with open(get_ssl_properties_file(config_name), mode='w', encoding='utf-8') as file:
         print('STORE_TYPE=JKS', file=file)
         print(f'FILE_PATH={get_projector_jks_file(config_name)}', file=file)
         print(f'STORE_PASSWORD={token}', file=file)
@@ -200,7 +320,13 @@ def generate_ssl_properties_file(config_name: str, token: str) -> None:
 
 def get_keytool(path_to_app: str) -> str:
     """Returns full path to keytool for given config"""
-    return join(get_jre_dir(path_to_app), 'bin', 'keytool')
+
+    if is_linux_x86_64():
+        return join(get_jre_dir(path_to_app), 'bin', 'keytool')
+
+    res = shutil.which('keytool')
+
+    return res if res else join(get_jre_dir(path_to_app), 'bin', 'keytool')
 
 
 def remove_server_secrets(config_name: str) -> None:
@@ -252,6 +378,7 @@ class SecureConfigGenerator:
     def _generate_ca(self) -> None:
         """Creates CA"""
         create_dir_if_not_exist(get_ssl_dir())
+        create_ca_ini(generate_token())
         self._run_keytool_with(get_generate_ca_command())
         self._run_keytool_with(get_export_ca_command())
 
