@@ -6,6 +6,9 @@
 """
 Misc utility functions.
 """
+import os
+import platform
+import stat
 import sys
 import io
 import json
@@ -14,9 +17,11 @@ import zipfile
 import subprocess
 import secrets
 import string
-from os import listdir, remove, makedirs
+from os import listdir, remove, makedirs, chmod
+
 from os.path import join, isfile, getsize, basename, isdir, realpath, expandvars, expanduser
 from shutil import copy
+from urllib.parse import ParseResult, urlparse
 from urllib.request import urlopen
 from typing import Optional, BinaryIO, cast, List, Any
 
@@ -27,6 +32,7 @@ CHUNK_SIZE = 4 * 1024 * 1024
 PROGRESS_BAR_WIDTH = 50
 PROGRESS_BAR_TEMPLATE = '[%(bar)s]  %(info)s'
 DEF_TOKEN_LEN = 20
+DOCKER_VENDOR = '02:42'
 
 
 def create_dir_if_not_exist(dir_name: str) -> None:
@@ -72,14 +78,16 @@ def download_file(url: str, destination: str, timeout: Optional[int] = None,
     """
     file_name = get_file_name_from_url(url)
     file_path = join(destination, file_name)
+    parsed_url: ParseResult = urlparse(url)
 
     with urlopen(url, timeout=timeout) as resp:
         code: int = resp.getcode()
 
-        if code != 200:
+        if parsed_url.scheme != 'file' and code != 200:
             raise IOError(f'Bad HTTP response code: {code}')
 
-        total = int(resp.getheader('Content-Length'))
+        total = int(resp.getheader('Content-Length')) if parsed_url.scheme != 'file' \
+            else os.path.getsize(parsed_url.path)
 
         if not isfile(file_path) or getsize(file_path) != total:
 
@@ -105,6 +113,15 @@ def download_file(url: str, destination: str, timeout: Optional[int] = None,
     return file_path
 
 
+def ensure_writable(path: str) -> None:
+    """Makes file writable by owner"""
+    if isfile(path):
+        file_stats = os.stat(path)
+
+        if (file_stats.st_mode & stat.S_IWUSR) == 0:
+            chmod(path, file_stats.st_mode | stat.S_IWUSR)
+
+
 def unpack_tar_file(file_path: str, destination: str) -> str:
     """ Unpacks given file in destination directory. """
     print(f'Unpacking {basename(file_path)}')
@@ -116,7 +133,9 @@ def unpack_tar_file(file_path: str, destination: str) -> str:
         with progressbar(length=len(members), width=PROGRESS_BAR_WIDTH,
                          bar_template=PROGRESS_BAR_TEMPLATE) as progress_bar:
             for member in members:
-                tar_file.extract(member, destination)
+                out_member_path = join(destination, member.name)
+                tar_file.extract(member=member, path=destination)
+                ensure_writable(out_member_path)  # workaround for MPS licenses
                 progress_bar.update(1)
 
     return dir_name
@@ -146,17 +165,39 @@ def get_java_version(java_path: str) -> str:
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
     line = io.TextIOWrapper(cast(BinaryIO, proc.stderr), encoding='utf-8').readline()
+    proc.wait()
     values = line.split(' ')
     version = values[2]
     return version.strip('"')
 
 
+def is_inside_docker() -> bool:
+    """Detects if we run inside docker container"""
+    return isfile('/.dockerenv')
+
+
+def is_docker_interface(ifs: Any) -> bool:
+    """Returns True if given interface belongs to docker"""
+    addresses = netifaces.ifaddresses(ifs)
+
+    if netifaces.AF_LINK in addresses:
+        for mac in addresses[netifaces.AF_LINK]:
+            if mac['addr'][:5] == DOCKER_VENDOR:
+                return True
+
+    return False
+
+
 def get_local_addresses() -> List[str]:
     """Returns list of local ip addresses."""
     interfaces = netifaces.interfaces()
     res = []
 
     for ifs in interfaces:
+
+        if not is_inside_docker() and is_docker_interface(ifs):
+            continue
+
         addresses = netifaces.ifaddresses(ifs)
 
         if netifaces.AF_INET in addresses:
@@ -188,7 +229,7 @@ def generate_token(length: int = DEF_TOKEN_LEN) -> str:
 def get_distributive_name() -> str:
     """Try to obtain distributive name from /etc/lsb-release"""
     try:
-        with open('/etc/lsb-release', 'r') as file:
+        with open('/etc/lsb-release', mode='r', encoding='utf-8') as file:
             for line in file:
                 if line.startswith('DISTRIB_ID'):
                     parts = line.split('=')
@@ -215,3 +256,8 @@ def is_in_venv() -> bool:
         return getattr(sys, "base_prefix", None) or getattr(sys, "real_prefix", None) or sys.prefix
 
     return get_base_prefix() != sys.prefix
+
+
+def is_linux_x86_64() -> bool:
+    """Returns true for Linux x86_64 machine"""
+    return platform.system() == 'Linux' and platform.machine() == 'x86_64'
